import { v4 as uuidv4 } from 'uuid';
// Types of background processes we can track
export var BackgroundToolType;
(function (BackgroundToolType) {
    BackgroundToolType["SHELL"] = "shell";
    BackgroundToolType["BROWSER"] = "browser";
    BackgroundToolType["AGENT"] = "agent";
})(BackgroundToolType || (BackgroundToolType = {}));
// Status of a background process
export var BackgroundToolStatus;
(function (BackgroundToolStatus) {
    BackgroundToolStatus["RUNNING"] = "running";
    BackgroundToolStatus["COMPLETED"] = "completed";
    BackgroundToolStatus["ERROR"] = "error";
    BackgroundToolStatus["TERMINATED"] = "terminated";
})(BackgroundToolStatus || (BackgroundToolStatus = {}));
/**
 * Registry to keep track of all background processes
 */
export class BackgroundToolRegistry {
    static instance;
    tools = new Map();
    // Private constructor for singleton pattern
    constructor() { }
    // Get the singleton instance
    static getInstance() {
        if (!BackgroundToolRegistry.instance) {
            BackgroundToolRegistry.instance = new BackgroundToolRegistry();
        }
        return BackgroundToolRegistry.instance;
    }
    // Register a new shell process
    registerShell(agentId, command) {
        const id = uuidv4();
        const tool = {
            id,
            type: BackgroundToolType.SHELL,
            status: BackgroundToolStatus.RUNNING,
            startTime: new Date(),
            agentId,
            metadata: {
                command,
            },
        };
        this.tools.set(id, tool);
        return id;
    }
    // Register a new browser process
    registerBrowser(agentId, url) {
        const id = uuidv4();
        const tool = {
            id,
            type: BackgroundToolType.BROWSER,
            status: BackgroundToolStatus.RUNNING,
            startTime: new Date(),
            agentId,
            metadata: {
                url,
            },
        };
        this.tools.set(id, tool);
        return id;
    }
    // Register a new agent process (for future use)
    registerAgent(agentId, goal) {
        const id = uuidv4();
        const tool = {
            id,
            type: BackgroundToolType.AGENT,
            status: BackgroundToolStatus.RUNNING,
            startTime: new Date(),
            agentId,
            metadata: {
                goal,
            },
        };
        this.tools.set(id, tool);
        return id;
    }
    // Update the status of a process
    updateToolStatus(id, status, metadata) {
        const tool = this.tools.get(id);
        if (!tool) {
            return false;
        }
        tool.status = status;
        if (status === BackgroundToolStatus.COMPLETED ||
            status === BackgroundToolStatus.ERROR ||
            status === BackgroundToolStatus.TERMINATED) {
            tool.endTime = new Date();
        }
        if (metadata) {
            tool.metadata = { ...tool.metadata, ...metadata };
        }
        return true;
    }
    // Get all processes for a specific agent
    getToolsByAgent(agentId) {
        const result = [];
        for (const tool of this.tools.values()) {
            if (tool.agentId === agentId) {
                result.push(tool);
            }
        }
        return result;
    }
    // Get a specific process by ID
    getToolById(id) {
        return this.tools.get(id);
    }
    // Clean up completed processes (optional, for maintenance)
    cleanupOldTools(olderThanHours = 24) {
        const cutoffTime = new Date(Date.now() - olderThanHours * 60 * 60 * 1000);
        for (const [id, tool] of this.tools.entries()) {
            // Remove if it's completed/error/terminated AND older than cutoff
            if (tool.endTime &&
                tool.endTime < cutoffTime &&
                (tool.status === BackgroundToolStatus.COMPLETED ||
                    tool.status === BackgroundToolStatus.ERROR ||
                    tool.status === BackgroundToolStatus.TERMINATED)) {
                this.tools.delete(id);
            }
        }
    }
}
// Export singleton instance
export const backgroundToolRegistry = BackgroundToolRegistry.getInstance();
//# sourceMappingURL=backgroundTools.js.map